@@ -14,11 +14,11 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
-* $Id: check_hpjd.c,v 1.31 2004/12/25 23:17:44 opensides Exp $
+* $Id: check_hpjd.c,v 1.32 2005/06/27 13:07:39 seanius Exp $
 *****************************************************************************/
 
 const char *progname = "check_hpjd";
-const char *revision = "$Revision: 1.31 $";
+const char *revision = "$Revision: 1.32 $";
 const char *copyright = "2000-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -36,8 +36,8 @@ const char *option_summary = "-H host [-C community]\n";
 #define HPJD_PAPER_STATUS          ".1.3.6.1.4.1.11.2.3.9.1.1.2.2"
 #define HPJD_INTERVENTION_REQUIRED ".1.3.6.1.4.1.11.2.3.9.1.1.2.3"
 #define HPJD_GD_PERIPHERAL_ERROR   ".1.3.6.1.4.1.11.2.3.9.1.1.2.6"
-#define HPJD_GD_PAPER_JAM          ".1.3.6.1.4.1.11.2.3.9.1.1.2.8"
-#define HPJD_GD_PAPER_OUT          ".1.3.6.1.4.1.11.2.3.9.1.1.2.9"
+#define HPJD_GD_PAPER_OUT          ".1.3.6.1.4.1.11.2.3.9.1.1.2.8"
+#define HPJD_GD_PAPER_JAM          ".1.3.6.1.4.1.11.2.3.9.1.1.2.9"
 #define HPJD_GD_TONER_LOW          ".1.3.6.1.4.1.11.2.3.9.1.1.2.10"
 #define HPJD_GD_PAGE_PUNT          ".1.3.6.1.4.1.11.2.3.9.1.1.2.11"
 #define HPJD_GD_MEMORY_OUT         ".1.3.6.1.4.1.11.2.3.9.1.1.2.12"
@@ -134,7 +134,7 @@ main (int argc, char **argv)
 		temp_buffer = strtok (input_buffer, "=");
 		temp_buffer = strtok (NULL, "=");
 
-		if (temp_buffer == NULL) {
+		if (temp_buffer == NULL && line < 13) {
 
 				result = STATE_UNKNOWN;
 				strcpy (errmsg, input_buffer);
@@ -179,8 +179,9 @@ main (int argc, char **argv)
 			case 12:										/* display panel message */
 				strcpy (display_message, temp_buffer + 1);
 				break;
-			default:
-				break;
+			default:										/* fold multiline message */
+				strncat (display_message, input_buffer, 
+						sizeof (display_message) - strlen (display_message) - 1);
 			}
 
 		}
