@@ -14,12 +14,12 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Id: check_snmp.c,v 1.53 2005/01/21 01:03:56 harpermann Exp $
+ $Id: check_snmp.c,v 1.57 2005/06/01 19:41:01 sghosh Exp $
  
 ******************************************************************************/
 
 const char *progname = "check_snmp";
-const char *revision = "$Revision: 1.53 $";
+const char *revision = "$Revision: 1.57 $";
 const char *copyright = "1999-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -82,6 +82,7 @@ regex_t preg;
 regmatch_t pmatch[10];
 char timestamp[10] = "";
 char errbuf[MAX_INPUT_BUFFER];
+char perfstr[MAX_INPUT_BUFFER];
 int cflags = REG_EXTENDED | REG_NOSUB | REG_NEWLINE;
 int eflags = 0;
 int errcode, excode;
@@ -108,6 +109,7 @@ size_t labels_size = 8;
 size_t nunits = 0;
 size_t unitv_size = 8;
 int verbose = FALSE;
+int usesnmpgetnext = FALSE;
 unsigned long lower_warn_lim[MAX_OIDS];
 unsigned long upper_warn_lim[MAX_OIDS];
 unsigned long lower_crit_lim[MAX_OIDS];
@@ -119,7 +121,9 @@ int retries = 0;
 unsigned long eval_method[MAX_OIDS];
 char *delimiter;
 char *output_delim;
-char *miblist;
+char *miblist = NULL;
+int needmibs = FALSE;
+
 
 int
 main (int argc, char **argv)
@@ -136,6 +140,7 @@ main (int argc, char **argv)
 	char *ptr = NULL;
 	char *p2 = NULL;
 	char *show = NULL;
+	char type[8];
 
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, LOCALEDIR);
@@ -153,9 +158,9 @@ main (int argc, char **argv)
 	port = strdup (DEFAULT_PORT);
 	outbuff = strdup ("");
 	output = strdup ("");
-	delimiter = strdup (DEFAULT_DELIMITER);
+	delimiter = strdup (" = ");
 	output_delim = strdup (DEFAULT_OUTPUT_DELIMITER);
-	miblist = strdup (DEFAULT_MIBLIST);
+	/* miblist = strdup (DEFAULT_MIBLIST); */
 	timeout_interval = DEFAULT_TIMEOUT;
 	retries = DEFAULT_RETRIES;
 
@@ -163,11 +168,20 @@ main (int argc, char **argv)
 		usage4 (_("Could not parse arguments"));
 
 	/* create the command line to execute */
-	asprintf (&command_line, "%s -t %d -r %d -m %s -v %s %s %s:%s %s",
+		if(usesnmpgetnext == TRUE) {
+		asprintf(&command_line, "%s -t %d -r %d -m %s -v %s %s %s:%s %s",
+				    PATH_TO_SNMPGETNEXT, timeout_interval, retries, miblist, proto,
+						authpriv, server_address, port, oid);
+	}else{
+
+		asprintf (&command_line, "%s -t %d -r %d -m %s -v %s %s %s:%s %s",
 	          PATH_TO_SNMPGET, timeout_interval, retries, miblist, proto,
 	          authpriv, server_address, port, oid);
+	}
+	
 	if (verbose)
 		printf ("%s\n", command_line);
+	
 
 	/* run the command */
 	child_process = spopen (command_line);
@@ -189,9 +203,14 @@ main (int argc, char **argv)
 
 	ptr = output;
 
+	strcat(perfstr, "| ");
 	while (ptr) {
+		char *foo;
+
+		foo = strstr (ptr, delimiter);
+		strncat(perfstr, ptr, foo-ptr);
+		ptr = foo; 
 
-		ptr = strstr (ptr, delimiter);
 		if (ptr == NULL)
 			break;
 
@@ -227,8 +246,10 @@ main (int argc, char **argv)
 			show = strstr (response, "Gauge: ") + 7;
 		else if (strstr (response, "Gauge32: "))
 			show = strstr (response, "Gauge32: ") + 9;
-		else if (strstr (response, "Counter32: "))
+		else if (strstr (response, "Counter32: ")) {
 			show = strstr (response, "Counter32: ") + 11;
+			strcpy(type, "c");
+		}
 		else if (strstr (response, "INTEGER: "))
 			show = strstr (response, "INTEGER: ") + 9;
 		else if (strstr (response, "STRING: "))
@@ -317,7 +338,11 @@ main (int argc, char **argv)
 
 		i++;
 
-	}															/* end while (ptr) */
+		char *str[MAX_INPUT_BUFFER];
+		asprintf(str, "=%s%s;;;; ", show, type ? type : "");
+		strcat(perfstr, *str);
+
+	}	/* end while (ptr) */
 
 	if (found == 0)
 		die (STATE_UNKNOWN,
@@ -339,7 +364,7 @@ main (int argc, char **argv)
 /* 	if (nunits == 1 || i == 1) */
 /* 		printf ("%s %s -%s %s\n", label, state_text (result), outbuff, units); */
 /* 	else */
-	printf ("%s %s -%s\n", label, state_text (result), outbuff);
+	printf ("%s %s -%s %s \n", label, state_text (result), outbuff, perfstr);
 
 	return result;
 }
@@ -378,6 +403,7 @@ process_arguments (int argc, char **argv)
 		{"authproto", required_argument, 0, 'a'},
 		{"authpasswd", required_argument, 0, 'A'},
 		{"privpasswd", required_argument, 0, 'X'},
+		{"next", no_argument, 0, 'n'},
 		{0, 0, 0, 0}
 	};
 
@@ -395,7 +421,7 @@ process_arguments (int argc, char **argv)
 	}
 
 	while (1) {
-		c = getopt_long (argc, argv, "hvVt:c:w:H:C:o:e:E:d:D:s:t:R:r:l:u:p:m:P:L:U:a:A:X:",
+		c = getopt_long (argc, argv, "nhvVt:c:w:H:C:o:e:E:d:D:s:t:R:r:l:u:p:m:P:L:U:a:A:X:",
 									 longopts, &option);
 
 		if (c == -1 || c == EOF)
@@ -427,6 +453,9 @@ process_arguments (int argc, char **argv)
 		case 'm':      /* List of MIBS  */
 			miblist = optarg;
 			break;
+		case 'n':     /* usesnmpgetnext */
+			usesnmpgetnext = TRUE;
+			break;
 		case 'P':     /* SNMP protocol version */
 			proto = optarg;
 			break;
@@ -483,6 +512,15 @@ process_arguments (int argc, char **argv)
 				retries = atoi(optarg);
 			break;
 		case 'o':									/* object identifier */
+			if ( strspn( optarg, "0123456789." ) != strlen( optarg ) ) {
+					/*
+					 * we have something other than digits and periods, so we
+					 * have a mib variable, rather than just an SNMP OID, so
+					 * we have to actually read the mib files
+					 */
+					needmibs = TRUE;
+			}
+
 			for (ptr = optarg; (ptr = index (ptr, ',')); ptr++)
 				ptr[0] = ' '; /* relpace comma with space */
 			for (ptr = optarg; (ptr = index (ptr, ' ')); ptr++)
@@ -600,6 +638,8 @@ process_arguments (int argc, char **argv)
 
 	if (community == NULL)
 		community = strdup (DEFAULT_COMMUNITY);
+	
+
 
 	return validate_arguments ();
 }
@@ -613,13 +653,11 @@ process_arguments (int argc, char **argv)
 
 <para>&PROTO_validate_arguments;</para>
 
-<para>Given a database name, this function returns TRUE if the string
-is a valid PostgreSQL database name, and returns false if it is
-not.</para>
+<para>Checks to see if the default miblist needs to be loaded. Also verifies 
+the authentication and authorization combinations based on protocol version 
+selected.</para>
 
-<para>Valid PostgreSQL database names are less than &NAMEDATALEN;
-characters long and consist of letters, numbers, and underscores. The
-first character cannot be a number, however.</para>
+<para></para>
 
 </sect3>
 -@@
@@ -630,6 +668,15 @@ first character cannot be a number, however.</para>
 int
 validate_arguments ()
 {
+	/* check whether to load locally installed MIBS (CPU/disk intensive) */
+	if (miblist == NULL) {
+		if ( needmibs  == TRUE ) {
+			miblist = strdup (DEFAULT_MIBLIST);
+		}else{
+			miblist = "''";			/* don't read any mib files for numeric oids */
+		}
+	}
+
 
 	/* Need better checks to verify seclevel and authproto choices */
 	
@@ -646,6 +693,9 @@ validate_arguments ()
 		asprintf(&proto, DEFAULT_PROTOCOL);
 		asprintf(&authpriv, "%s%s", "-c ", community);
 	}
+	else if ( strcmp (proto, "2c") == 0 ) {                 /* snmpv2c args */
+		asprintf(&authpriv, "%s%s", "-c ", community);
+	}
 	else if ( strcmp (proto, "3") == 0 ) {                 /* snmpv3 args */
 		asprintf(&proto, "%s", "3");
 		
@@ -859,7 +909,9 @@ Check status of remote machines and obtain sustem information via SNMP\n\n"));
 
 	/* SNMP and Authentication Protocol */
 	printf (_("\
- -P, --protocol=[1|3]\n\
+ -n, --next\n\
+    Use SNMP GETNEXT instead of SNMP GET\n\
+ -P, --protocol=[1|2c|3]\n\
     SNMP protocol version\n\
  -L, --seclevel=[noAuthNoPriv|authNoPriv|authPriv]\n\
     SNMPv3 securityLevel\n\
@@ -881,9 +933,11 @@ Check status of remote machines and obtain sustem information via SNMP\n\n"));
 	/* OID Stuff */
 	printf (_("\
  -o, --oid=OID(s)\n\
-    Object identifier(s) whose value you wish to query\n\
+    Object identifier(s) or SNMP variables whose value you wish to query\n\
  -m, --miblist=STRING\n\
-    List of MIBS to be loaded (default = ALL)\n -d, --delimiter=STRING\n\
+    List of MIBS to be loaded (default = none if using numeric oids or 'ALL'\n\
+		for symbolic oids.)\n\
+ -d, --delimiter=STRING\n\
     Delimiter to use when parsing returned data. Default is \"%s\"\n\
     Any data on the right hand side of the delimiter is considered\n\
     to be the data that should be used in the evaluation.\n"), DEFAULT_DELIMITER);
