@@ -21,12 +21,12 @@
  *
  * Copyright (c) 2000 Karl DeBisschop (kdebisschop@users.sourceforge.net)
  *
- * $Id: check_swap.c,v 1.51 2005/07/06 22:10:04 tonvoon Exp $
+ * $Id: check_swap.c,v 1.52 2005/09/15 08:27:58 tonvoon Exp $
  *
  *****************************************************************************/
 
 const char *progname = "check_swap";
-const char *revision = "$Revision: 1.51 $";
+const char *revision = "$Revision: 1.52 $";
 const char *copyright = "2000-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -55,7 +55,6 @@ main (int argc, char **argv)
 	float dsktotal = 0, dskused = 0, dskfree = 0, tmp = 0;
 	int result = STATE_UNKNOWN;
 	char input_buffer[MAX_INPUT_BUFFER];
-	char *perf;
 #ifdef HAVE_PROC_MEMINFO
 	FILE *fp;
 #else
@@ -86,7 +85,6 @@ main (int argc, char **argv)
 	textdomain (PACKAGE);
 
 	status = strdup ("");
-	perf = strdup ("");
 
 	if (process_arguments (argc, argv) == ERROR)
 		usage4 (_("Could not parse arguments"));
