@@ -23,12 +23,12 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Id: check_ups.c,v 1.26 2004/12/25 23:17:44 opensides Exp $
+ $Id: check_ups.c,v 1.27 2005/03/04 21:58:40 tonvoon Exp $
  
 ******************************************************************************/
 
 const char *progname = "check_ups";
-const char *revision = "$Revision: 1.26 $";
+const char *revision = "$Revision: 1.27 $";
 const char *copyright = "2000-2004";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -408,7 +408,7 @@ get_ups_variable (const char *varname, char *buf, size_t buflen)
 	}
 
 	if (strcmp (ptr, "ERR VAR-NOT-SUPPORTED") == 0) {
-		//printf ("Error: Variable '%s' is not supported\n", varname);
+		/*printf ("Error: Variable '%s' is not supported\n", varname);*/
 		return NOSUCHVAR;
 	}
 
